import 'dart:async';
import 'dart:convert';
import 'dart:io';

import 'package:apimodule/msimpersaa/request/issue_request.dart';
import 'package:apimodule/msimpersaa/response/add_file_response.dart';
import 'package:path/path.dart';
import 'package:async/async.dart';

import 'package:apimodule/msimpersaa/request/efile_request.dart';
import 'package:apimodule/msimpersaa/response/efile_response.dart';
import 'package:apimodule/msimpersaa/response/jenis_file_response.dart';
import 'package:http/http.dart' as http;
import 'package:apimodule/api/config/endpoint.dart';
import 'package:apimodule/api/config/apisettings.dart';
import 'package:apimodule/msimpersaa/request/global_request.dart';
import 'package:apimodule/msimpersaa/response/global_response.dart';

class EfileService {
  static Future<dynamic> fetchAllFile(GlobalRequest request) async {
    var globalRequest = new Map<String, dynamic>();
    globalRequest['nip_nik'] = request.nipNik;

    try {
      final response = await http.post(
        EndpointSimpersaa.getAllFile,
        headers: null,
        body: globalRequest,
      );
      if (response.statusCode == 200) {
        final body = jsonDecode(response.body);
        return EfileResponse.fromJson(body);
      } else {
        return GlobalResponse(msg: APiSettings.errorMsg);
      }
    } on TimeoutException catch (_) {
      return GlobalResponse(msg: APiSettings.errorNetwork);
    } on SocketException catch (_) {
      return GlobalResponse(msg: APiSettings.errorNetwork);
    }
  }

  static Future<dynamic> fetchAllJenisFile(GlobalRequest request) async {
    var globalRequest = new Map<String, dynamic>();
    globalRequest['nip_nik'] = request.nipNik;

    try {
      final response = await http.post(
        EndpointSimpersaa.getAllJenisFile,
        headers: null,
        body: globalRequest,
      );
      if (response.statusCode == 200) {
        final body = jsonDecode(response.body);
        return JenisFileResponse.fromJson(body);
      } else {
        return GlobalResponse(msg: APiSettings.errorMsg);
      }
    } on TimeoutException catch (_) {
      return GlobalResponse(msg: APiSettings.errorNetwork);
    } on SocketException catch (_) {
      return GlobalResponse(msg: APiSettings.errorNetwork);
    }
  }

  static Future<dynamic> fetchAddFile(IssueKeteranganRequest request) async {
    var globalRequest = new Map<String, dynamic>();
    globalRequest['nip_nik'] = request.nipNik;
    globalRequest['jenis_issue'] = request.jenisIssue;
    globalRequest['nomor'] = request.nomor;
    globalRequest['tanggal_terbit'] = request.tanggalTerbit;
    globalRequest['penerbit'] = request.penerbit;

    try {
      final response = await http.post(
        EndpointSimpersaa.addFiles,
        headers: null,
        body: globalRequest,
      );
      if (response.statusCode == 200) {
        final body = jsonDecode(response.body);
        return AddFileResponse.fromJson(body);
      } else {
        return GlobalResponse(msg: APiSettings.errorMsg);
      }
    } on TimeoutException catch (_) {
      return GlobalResponse(msg: APiSettings.errorNetwork);
    } on SocketException catch (_) {
      return GlobalResponse(msg: APiSettings.errorNetwork);
    }
  }

  static Future<http.StreamedResponse> fetchUploadFile(
      EfileRequest efileRequest) async {
    var stream = http.ByteStream(
        // ignore: deprecated_member_use
        DelegatingStream.typed(efileRequest.dokumen.openRead()));
    var length = await efileRequest.dokumen.length();
    var request = http.MultipartRequest("POST", EndpointSimpersaa.uploadFile);

    var multipartFile = http.MultipartFile('dokumen', stream, length,
        filename: basename(efileRequest.dokumen.path));

    request.files.add(multipartFile);
    request.fields['nip_nik'] = efileRequest.nipNik;
    request.fields['id'] = efileRequest.id;
    request.fields['document_name'] = efileRequest.documentName;
    request.fields['path'] = efileRequest.path;

    var response = await request.send();
    return response;
  }
}
